"""
This is the autogamess plotting module


Author: Brian C. Ferrari
"""
from ..config                  import *


from .make_kde                 import make_kde
from .make_scatter             import make_scatter
from .prep_data                import prep_data
